@@ -21,7 +21,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.18';
+$VERSION = '1.19';
 
 sub ProcessPEResources($$);
 sub ProcessPEVersion($$);
@@ -726,6 +726,7 @@ my %languageCode = (
         Name => 'CPUType',
         Format => 'int16u',
         # ref /usr/include/linux/elf-em.h
+        # ref https://en.wikipedia.org/wiki/Executable_and_Linkable_Format
         PrintConv => {
             0 => 'None',
             1 => 'AT&T WE 32100',
@@ -736,21 +737,68 @@ my %languageCode = (
             6 => 'i486',
             7 => 'i860',
             8 => 'MIPS R3000',
+            9 => 'IBM System/370',
             10 => 'MIPS R4000',
-            15 => 'HPPA',
+            15 => 'HP PA-RISC',
             18 => 'Sun v8plus',
+            19 => 'Intel 80960',
             20 => 'PowerPC',
             21 => 'PowerPC 64-bit',
             22 => 'IBM S/390',
             23 => 'Cell BE SPU',
+            36 => 'NEC V800',
+            37=> 'Fujitsu FR20',
+            38 => 'TRW RH-32',
+            39 => 'Motorola RCE',
+            40 => 'Arm (up to Armv7/AArch32)',
+            41 => 'Digital Alpha',
             42 => 'SuperH',
             43 => 'SPARC v9 64-bit',
+            44 => 'Siemens TriCore',
+            45 => 'Argonaut RISC Core',
             46 => 'Renesas H8/300,300H,H8S',
+            47 => 'Hitachi H8/300H',
+            48 => 'Hitachi H8S',
+            49 => 'Hitachi H8/500',
             50 => 'HP/Intel IA-64',
-            62 => 'AMD x86-64',
-            76 => 'Axis Communications 32-bit embedded processor',
-            87 => 'NEC v850',
-            88 => 'Renesas M32R',
+            0x33 => 'Stanford MIPS-X',
+            0x34 => 'Motorola ColdFire',
+            0x35 => 'Motorola M68HC12',
+            0x36 => 'Fujitsu MMA Multimedia Accelerator',
+            0x37 => 'Siemens PCP',
+            0x38 => 'Sony nCPU embedded RISC processor',
+            0x39 => 'Denso NDR1 microprocessor',
+            0x3a => 'Motorola Star*Core processor',
+            0x3b => 'Toyota ME16 processor',
+            0x3c => 'STMicroelectronics ST100 processor',
+            0x3d => 'Advanced Logic Corp. TinyJ embedded processor family',
+            0x3e => 'AMD x86-64',
+            0x3f => 'Sony DSP Processor',
+            0x40 => 'Digital Equipment Corp. PDP-10',
+            0x41 => 'Digital Equipment Corp. PDP-11',
+            0x42 => 'Siemens FX66 microcontroller',
+            0x43 => 'STMicroelectronics ST9+ 8/16 bit microcontroller',
+            0x44 => 'STMicroelectronics ST7 8-bit microcontroller',
+            0x45 => 'Motorola MC68HC16 Microcontroller',
+            0x46 => 'Motorola MC68HC11 Microcontroller',
+            0x47 => 'Motorola MC68HC08 Microcontroller',
+            0x48 => 'Motorola MC68HC05 Microcontroller',
+            0x49 => 'Silicon Graphics SVx',
+            0x4a => 'STMicroelectronics ST19 8-bit microcontroller',
+            0x4b => 'Digital VAX',
+            0x4c => 'Axis Communications 32-bit embedded processor',
+            0x4d => 'Infineon Technologies 32-bit embedded processor',
+            0x4e => 'Element 14 64-bit DSP Processor',
+            0x4f => 'LSI Logic 16-bit DSP Processor',
+            0x57 => 'NEC v850',
+            0x58 => 'Renesas M32R',
+            0x8c => 'TMS320C6000 Family',
+            0xaf => 'MCST Elbrus e2k',
+            0xb7 => 'Arm 64-bits (Armv8/AArch64)',
+            0xdc => 'Zilog Z80',
+            0xf3 => 'RISC-V',
+            0xf7 => 'Berkeley Packet Filter',
+            0x101 => 'WDC 65C816',
             0x5441 => 'Fujitsu FR-V',
             0x9026 => 'Alpha', # (interim value)
             0x9041 => 'm32r (old)',
@@ -1406,7 +1454,7 @@ library files.
 
 =head1 AUTHOR
 
-Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
