@@ -20,7 +20,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.17';
+$VERSION = '1.20';
 
 # road map of directory locations in GIF images
 my %gifMap = (
@@ -54,6 +54,7 @@ my %gifMap = (
     Extensions => { # (for documentation only)
         SubDirectory => { TagTable => 'Image::ExifTool::GIF::Extensions' },
     },
+    TransparentColor => { },
 );
 
 # GIF89a application extensions:
@@ -84,6 +85,10 @@ my %gifMap = (
         Groups => { 2 => 'Audio' },
         Binary => 1,
     },
+    'C2PA_GIF/' => { #https://c2pa.org/specifications/ (NC) (authentication code is 0x010000 binary, so removed from tag ID)
+        Name => 'JUMBF',
+        SubDirectory => { TagTable => 'Image::ExifTool::Jpeg2000::Main' },
+    },
 );
 
 # GIF locical screen descriptor
@@ -115,6 +120,11 @@ my %gifMap = (
         ValueConv => '$val + 1',
     },
     5 => 'BackgroundColor',
+    6 => {
+        Name => 'PixelAspectRatio',
+        RawConv => '$val ? $val : undef',
+        ValueConv => '($val + 15) / 64',
+    },
 );
 
 # GIF Netscape 2.0 animation extension (ref 3)
@@ -466,10 +476,13 @@ Block:
         } elsif ($a == 0xf9 and $length == 4) {     # graphic control extension
 
             last unless $raf->Read($buff, $length) == $length;
-            # sum the indivual delay times
+            # sum the individual delay times
             my $delay = Get16u(\$buff, 1);
             $delayTime += $delay;
             $verbose and printf $out "Graphic Control: delay=%.2f\n", $delay / 100;
+            # get transparent colour
+            my $bits = Get8u(\$buff, 0);
+            $et->HandleTag($tagTablePtr, 'TransparentColor', Get8u(\$buff,3)) if $bits & 0x01;
             $raf->Seek(-$length, 1) or last;
 
         } elsif ($a == 0x01 and $length == 12) {    # plain text extension
@@ -532,7 +545,7 @@ write GIF meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
